<?php

namespace Drupal\oidc_menu;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Link;

/**
 * Represents a list of links to be shown in the menu.
 */
class LinkList implements \Iterator {

  /**
   * The list title.
   *
   * @var string
   */
  protected $title;

  /**
   * The links in this list.
   *
   * Each entry is an associative array with a 'link' and 'weight' element.
   *
   * @var array
   */
  protected $links = [];

  /**
   * Wether the links are sorted by weight.
   *
   * @var bool
   */
  protected $sorted = TRUE;

  /**
   * Default weight for the next link.
   *
   * @var int
   */
  protected $nextWeight = 0;

  /**
   * Class constructor.
   *
   * @param string $title
   *   The list title.
   */
  public function __construct($title = NULL) {
    $this->setTitle($title);
  }

  /**
   * Get the list title.
   *
   * @return string
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * Set the list title.
   *
   * @param string|null $title
   *   The new title.
   *
   * @return $this
   */
  public function setTitle($title = NULL) {
    if ($title === '') {
      $title = NULL;
    }

    $this->title = $title;

    return $this;
  }

  /**
   * Get all links.
   *
   * @return \Drupal\Core\Link[]
   *   All links in the list, sorted by weight.
   */
  public function getAll() {
    $this->sort();

    $links = [];
    foreach ($this->links as $index => $item) {
      $links[$index] = $item['link'];
    }

    return $links;
  }

  /**
   * Get a link.
   *
   * @param int $index
   *   The index of the link.
   *
   * @return \Drupal\Core\Link|null
   *   The link instance or NULL if the links doesn't exist.
   */
  public function get($index) {
    if (isset($this->links[$index])) {
      return $this->links[$index]['link'];
    }
  }

  /**
   * Add a link.
   *
   * @param \Drupal\Core\Link $link
   *   The link to add.
   * @param int|null $weight
   *   The link weight.
   *
   * @return $this
   */
  public function add(Link $link, $weight = NULL) {
    if ($weight === NULL) {
      $weight = $this->nextWeight;
      $this->nextWeight++;
    }

    $this->links[] = [
      'link' => $link,
      'weight' => $weight,
    ];

    if ($this->count() > 1) {
      $this->sorted = FALSE;
    }

    return $this;
  }

  /**
   * Remove a link.
   *
   * @param int $index
   *   The index of the link.
   *
   * @return $this
   */
  public function remove($index) {
    $index = (int) $index;

    if (isset($this->links[$index])) {
      unset($this->links[$index]);

      if ($this->count() === 1) {
        $this->sorted = TRUE;
      }
    }

    return $this;
  }

  /**
   * Remove a link.
   *
   * @param \Drupal\Core\Link $link
   *   The link object.
   * @param bool $multiple
   *   Set to TRUE to remove all instances (instead of only the first one).
   *
   * @return $this
   */
  public function removeByInstance(Link $link, $multiple = FALSE) {
    foreach ($this->links as $index => $item) {
      if ($item['link'] === $link) {
        $this->remove($index);

        if (!$multiple) {
          break;
        }
      }
    }

    return $this;
  }

  /**
   * Get the number of links.
   *
   * @return int
   */
  public function count() {
    return count($this->links);
  }

  /**
   * Sort the links.
   */
  protected function sort() {
    if (!$this->sorted) {
      uasort($this->links, [SortArray::class, 'sortByWeightElement']);

      $this->sorted = TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function current() {
    $this->sort();

    if ($item = current($this->links)) {
      return $item['link'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function next() {
    $this->sort();

    next($this->links);
  }

  /**
   * {@inheritdoc}
   */
  public function key() {
    $this->sort();

    return key($this->links);
  }

  /**
   * {@inheritdoc}
   */
  public function valid() {
    $this->sort();

    return $this->key() !== NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function rewind() {
    $this->sort();

    reset($this->links);
  }

}
